@@ -1,6 +1,12 @@
 import sys
+
 version = int(sys.version[0])
 is_python3 = version == 3
+JSONDecodeError = None
+if is_python3:
+    from json.decoder import JSONDecodeError
+else:
+    JSONDecodeError = ValueError
 
 
 def to_disasm(obj):
@@ -39,4 +45,4 @@ def ljust(source, size, by):
         raise Exception("Can't ljust data, size: {}, just: {}".format(len(source), by))
     source += by * length
     assert len(source) == size, "Error size: {}, len(source) = {}, by = {}".format(size, len(source), by)
-    return source
\ No newline at end of file
+    return source
