@@ -10,11 +10,14 @@ import lief
 from lief.ELF import SECTION_FLAGS
 from elf_to_shellcode.lib.utils.address_utils import AddressUtils
 from elf_to_shellcode.lib.utils.mini_loader import MiniLoader
-from elf_to_shellcode.lib.consts import StartFiles, OUTPUT_FORMAT_MAP, LoaderSupports
+from elf_to_shellcode.lib.consts import StartFiles, OUTPUT_FORMAT_MAP, LoaderSupports, Arches, ArchEndians, \
+    RELOCATION_OFFSETS, RelocationAttributes, HookTypes
 from elf_to_shellcode.lib.utils.disassembler import Disassembler
 from elf_to_shellcode.lib.ext.dynamic_symbols import DynamicRelocations
 from elf_to_shellcode.lib.utils.hooks import ShellcodeHooks
-from elf_to_shellcode.lib.utils.general import get_json, get_binary
+from elf_to_shellcode.lib.utils.general import get_binary
+from elf_to_shellcode.hooks.hooks_configuration_parser import HookConfiguration
+from elf_to_shellcode.hooks.base_hook import _BaseShelfHook
 
 PTR_SIZES = {
     4: "I",
@@ -90,37 +93,54 @@ class Shellcode(object):
 
         self.address_utils = AddressUtils(shellcode=self)
         self.mini_loader = MiniLoader(shellcode=self)
-        self.specific_arch_hook_configuration = {}
+        self.hooks_configuration = None
         if args.hooks_configuration:
             if not LoaderSupports.HOOKS in self.args.loader_supports:
                 raise Exception("Error hook configuration must be used with --loader-supports hooks")
-            self.resolve_specific_arch_hook_configuration()
+            self.hooks_configuration = HookConfiguration(args.hooks_configuration)
 
         if LoaderSupports.HOOKS in self.args.loader_supports:
             self.hooks = ShellcodeHooks(shellcode=self)
         else:
             self.hooks = None
 
-    def resolve_specific_arch_hook_configuration(self):
-        config = get_json(self.args.hooks_configuration)
-        if self.arch not in config:
-            raise Exception("Missing arch: {} in hook configuration".format(
-                self.arch
-            ))
-        arch_config = config[self.arch]
-        if self.args.endian not in arch_config:
-            raise Exception("Missing endian: {} in arch config, config[{}][{}]".format(
-                self.args.endian,
-                self.arch,
-                self.args.endian
-            ))
+        # Keep track of offsets inside the relocation table
+        self.offsets_in_header = {}
 
-        self.specific_arch_hook_configuration = arch_config[self.args.endian]
+    def _generic_do_hooks(self, hooks, hook_type):
+        self.logger.info("Adding hooks to: {}".format(hook_type))
+        for hook_cls in hooks:
+            hook = hook_cls(shellcode=self)
+            assert isinstance(hook, _BaseShelfHook)
+            self.hooks.add_hook(
+                shellcode_data=get_binary(hook.hook_get_shellcode_path(
+                    arch=Arches[self.args.arch],
+                    endian=ArchEndians[self.args.endian]
+                )),
+                hook_type=hook_type,
+                attributes=hook.hook_get_attributes()
+            )
 
     def do_hooks(self):
+        if not self.hooks_configuration:
+            return
         self.logger.info("Handling hooks")
-        for hook in self.specific_arch_hook_configuration.get('startup_hooks', []):
-            self.hooks.add_startup_hook(get_binary(hook))
+        self._generic_do_hooks(
+            hooks=self.hooks_configuration.startup_hooks,
+            hook_type=HookTypes.STARTUP_HOOKS
+        )
+        self._generic_do_hooks(
+            hooks=self.hooks_configuration.pre_relocate_write_hooks,
+            hook_type=HookTypes.PRE_RELOCATE_WRITE_HOOKS
+        )
+        self._generic_do_hooks(
+            hooks=self.hooks_configuration.pre_relocate_execute_hooks,
+            hook_type=HookTypes.PRE_RELOCATE_EXECUTE_HOOKS
+        )
+        self._generic_do_hooks(
+            hooks=self.hooks_configuration.pre_calling_main_shellcode_hooks,
+            hook_type=HookTypes.PRE_CALLING_MAIN_SHELLCODE_HOOKS
+        )
 
     def arch_find_relocation_handler(self, relocation_type):
         """
@@ -160,28 +180,42 @@ class Shellcode(object):
         else:
             raise NotImplementedError()
 
+    def pack_and_get_relocations_for_type(self, relocation_type):
+        packed = five.py_obj()
+        i = 0
+        for key, value in self.addresses_to_patch.items():
+            if type(value) is not list:
+                value = [value, RelocationAttributes.generic_relocate]
+            value, reloc_type = value
+            if reloc_type == relocation_type:
+                packed += self.address_utils.pack_pointers(key, value)
+                i += 1
+
+        return packed, i
+
     def relocation_table(self, padding=0x0):
         table = five.py_obj()
 
-        for key, value in self.addresses_to_patch.items():
-            if type(value) is not list:
-                value = [value]
-
-            value_packed = self.address_utils.pack_pointers(*value)
-
-            relocation_entry = self.address_utils.pack_pointer(key)
-            relocation_entry += value_packed
-
-            relocation_size = self.address_utils.pack_pointer(len(relocation_entry) + self.ptr_size)
-            relocation_entry = relocation_size + relocation_entry
-            table += relocation_entry
+        for relocation_type in RelocationAttributes:
+            packed, num_of_attributes = self.pack_and_get_relocations_for_type(relocation_type)
+            relocation_entry = self.mini_loader.structs.entry_attributes(
+                number_of_entries_related_to_attribute=num_of_attributes,
+                relocation_type=relocation_type.value
+            ).pack()
+            relocation_entry += packed
+            if num_of_attributes:
+                table += relocation_entry
 
         # Pack the following format: {size_t padding, size_t table_length, size_t header_length}
         sizes = self.address_utils.pack_pointers(padding,
                                                  len(table),
                                                  len(self.get_shellcode_header()))
 
-        header = self.address_utils.pack_pointer(self.shellcode_table_magic) + sizes + self.pre_table_header
+        header = self.address_utils.pack_pointer(self.shellcode_table_magic) + sizes
+        self.offsets_in_header[RELOCATION_OFFSETS.table_magic] = 0x0
+        self.offsets_in_header[RELOCATION_OFFSETS.padding_between_table_and_loader] = len(header)
+        header += self.address_utils.pack_pointer(0x0)  # padding_between_table_and_loader
+        header += self.pre_table_header
         if LoaderSupports.HOOKS in self.args.loader_supports:
             header += self.hooks.get_header()
         header += table
@@ -194,7 +228,13 @@ class Shellcode(object):
         header += self.address_utils.pack_pointer(
             self.elffile.header.e_ehsize + sht_entry_header_size
         )
-        header += self.address_utils.pack_pointer(len(self.mini_loader.loader))
+        if self.args.output_format != OUTPUT_FORMAT_MAP.eshelf:
+            header += self.address_utils.pack_pointer(len(self.mini_loader.loader))
+        else:
+            # In eshelf the elf is the loader therefor it is incorrect to add header size
+            header += self.address_utils.pack_pointer(0x0)
+        header += self.mini_loader.function_descriptor_header
+
         return header
 
     def correct_symbols(self, shellcode_data):
@@ -359,7 +399,7 @@ class Shellcode(object):
     def get_shellcode_header(self):
         original_entry_point = self.elffile.header.e_entry
         new_entry_point = (original_entry_point - self.loading_virtual_address)
-        return struct.pack("{}{}".format(self.endian, self.ptr_fmt), new_entry_point)
+        return self.address_utils.pack_pointer(new_entry_point)
 
     def build_shellcode_from_header_and_code(self, header, code):
         return header + code
@@ -387,6 +427,7 @@ class Shellcode(object):
                                      shellcode_data=shellcode_data)
         shellcode_data = self.do_objdump(shellcode_data)
         # Calling the do hooks
+
         self.do_hooks()
 
         # This must be here !
@@ -421,6 +462,7 @@ class Shellcode(object):
         segment.flags = rwx
         segment.content = bytearray(shellcode_data)
         segment = loader.add(segment)
+
         tmp_path = tempfile.mktemp(".out")
         elf_buffer = None
         try:
@@ -433,6 +475,22 @@ class Shellcode(object):
             if os.path.exists(tmp_path):
                 os.remove(tmp_path)
         assert elf_buffer is not None, "Error"
+        # Changing all offsets accordingly:
+        for off, value in self.offsets_in_header.items():
+            value += segment.file_offset
+            self.offsets_in_header[off] = value
+        loader_main_off = self.mini_loader.symbols.get_symbol_address("loader_main")
+        magic_off = self.offsets_in_header[RELOCATION_OFFSETS.table_magic]
+        padding_between_table_and_loader_off = self.offsets_in_header[
+            RELOCATION_OFFSETS.padding_between_table_and_loader]
+        # Checking the offset
+        assert elf_buffer[magic_off:magic_off + self.ptr_size] == self.address_utils.pack_pointer(
+            self.shellcode_table_magic), elf_buffer[magic_off:magic_off + self.ptr_size]
+        padding_between_table_and_loader = segment.virtual_address - loader_main_off
+        # Now replacing the padding_between_table_and_loader
+        elf_buffer_p1 = elf_buffer[:padding_between_table_and_loader_off]
+        elf_buffer_p2 = elf_buffer[padding_between_table_and_loader_off + self.ptr_size:]
+        elf_buffer = elf_buffer_p1 + self.address_utils.pack_pointer(padding_between_table_and_loader) + elf_buffer_p2
         loader_symbol_address = self.mini_loader.loader.find(self.address_utils.pack_pointer(0xdeadbeff))
         assert loader_symbol_address == self.mini_loader.loader.rfind(
             self.address_utils.pack_pointer(0xdeadbeff)), "Error found more then one " \
@@ -465,20 +523,6 @@ class Shellcode(object):
         return struct.unpack("{}{}".format(self.endian,
                                            self.ptr_fmt * num_of_ptrs), stream[:self.ptr_size * num_of_ptrs])
 
-    def get_loader_base_address(self, shellcode):
-        loader_size = len(self.mini_loader.loader)
-        table_length = len(self.relocation_table(0x0))
-        offset = loader_size + table_length
-        return self.unpack_ptr(shellcode[offset:offset + self.ptr_size])
-
-    def set_loader_base_address(self, shellcode, new_base_address):
-        loader_size = len(self.mini_loader.loader)
-        table_length = len(self.relocation_table(0x0))
-        offset = loader_size + table_length
-        shellcode = shellcode[:offset] + self.address_utils.pack_pointer(new_base_address) + shellcode[
-                                                                                             offset + self.ptr_size:]
-        return shellcode
-
     def embed(self, **kwargs):
         for key, value in kwargs.items():
             globals()[key] = value
