@@ -20,14 +20,14 @@ class Disassembler(object):
         mode = capstone.CS_MODE_32
         if shellcode.arch in ['x64']:
             mode = capstone.CS_MODE_64
-        if shellcode.arch in ["arm32", 'arm64']:
+        elif shellcode.arch in ["arm32", 'arm64']:
             mode = capstone.CS_MODE_ARM
         self.cs = capstone.Cs(
             ARCHES[shellcode.arch],
             ENDIAN[shellcode.endian] | mode
         )
         self.shellcode = shellcode
-        offset = self.shellcode.instruction_offset_after_objdump
+        offset = self.shellcode.linker_base_address
         self.opcodes = self.shellcode.do_objdump(self.shellcode.shellcode_data)[offset:]
         self.instructions = [instruction for instruction in self.cs.disasm(
             five.to_disasm(self.opcodes),
@@ -42,3 +42,10 @@ class Disassembler(object):
                 to_off -= self.shellcode.ptr_size
                 addresses.append(instruction.address + to_off)
         return addresses
+
+    def disassemble(self, code):
+        return self.cs.disasm(code, 0)
+
+    @staticmethod
+    def print_instruction(instruction):
+        print("0x%x:\t%s\t%s" % (instruction.address, instruction.mnemonic, instruction.op_str))
