@@ -128,3 +128,15 @@ class MiniLoader(object):
     @property
     def structs(self):
         return load_structs(self.structs_file)
+
+    @property
+    def function_descriptor_header(self):
+        functions = self.structs.loader_function_descriptor.__fields__
+        kwargs = {}
+        for function in functions:
+            kwargs[function] = self.symbols.get_relative_symbol_address(
+                function
+            )
+        return self.structs.loader_function_descriptor(
+                **kwargs
+            ).pack()
