@@ -3,32 +3,16 @@ from elf_to_shellcode.intel.x32 import intel_x32_make_shellcode
 from elf_to_shellcode.intel.x64 import intel_x64_make_shellcode
 from elf_to_shellcode.arm.x32 import arm_x32_make_shellcode
 from elf_to_shellcode.arm.x64 import arm_x64_make_shellcode
-from elf_to_shellcode.lib.consts import StartFiles
+from elf_to_shellcode.lib.consts import StartFiles, Arches, ArchEndians
 
-
-class Arches(object):
-    MIPS_32 = "mips"
-    INTEL_X32 = "intel_x32"
-    INTEL_X64 = "intel_x64"
-    ARM_32 = "arm32"
-    ARM_64 = "aarch64"
-    __all__ = [
-        MIPS_32,
-        INTEL_X32,
-        INTEL_X64,
-        ARM_32,
-        ARM_64
-    ]
-
-
-ENDIANS = ["big", "little"]
+ENDIANS = [ArchEndians.big.value, ArchEndians.little.value]
 
 shellcode_handlers = {
-    Arches.MIPS_32: mips_make_shellcode,
-    Arches.INTEL_X32: intel_x32_make_shellcode,
-    Arches.INTEL_X64: intel_x64_make_shellcode,
-    Arches.ARM_32: arm_x32_make_shellcode,
-    Arches.ARM_64: arm_x64_make_shellcode,
+    Arches.mips.value: mips_make_shellcode,
+    Arches.intel_x32.value: intel_x32_make_shellcode,
+    Arches.intel_x64.value: intel_x64_make_shellcode,
+    Arches.arm32.value: arm_x32_make_shellcode,
+    Arches.aarch64.value: arm_x64_make_shellcode,
 }
 
 
